from hivepy.template_management.models.project_template import ProjectTemplate
from hivepy.template_management.models.issue_template import IssueTemplate


__all__ = [
    'IssueTemplate',
    'ProjectTemplate',
]
