__description__ = "Small wrapper for HexWay Hive API "
__author__ = "@Cur1iosity"
__email__ = "Cur1iosity@protonmail.com"
__licence__ = "MIT"

from hivepy import rest
from hivepy.hive_api import HiveApi


__all__ = [
    'rest',
    'HiveApi',
]
